@@ -1,7 +1,7 @@
 /* mpfr_gmp -- Limited gmp-impl emulator
    Modified version of the GMP files.
 
-Copyright 2004, 2005 Free Software Foundation.
+Copyright 2004, 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of the MPFR Library.
 
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <stdlib.h> /* For malloc, free, realloc and abort*/
@@ -373,18 +373,3 @@ mpfr_default_free (void *blk_ptr, size_t blk_size)
 }
 
 #endif /* Have gmp-impl.h */
-
-#ifndef MPFR_HAVE_MPN_SUB_NC
-mp_limb_t
-mpfr_sub_nc (mp_ptr dest, mp_srcptr op1, mp_srcptr op2, mp_size_t s,
-             mp_limb_t c)
-{
-  mp_limb_t c2;
-  c2 = mpn_sub_n (dest, op1, op2, s);
-  MPFR_ASSERTD (c+c2 < MPFR_LIMB_HIGHBIT);
-  c2 = mpn_sub_1 (dest, dest, s, c+c2);
-  return c2;
-}
-
-#endif /* !Have MPFR_HAVE_MPN_SUB_NC */
-
