@@ -1,6 +1,6 @@
 /* mpfr_zeta -- compute the Riemann Zeta function
 
-Copyright 2003, 2004, 2005 Free Software Foundation.
+Copyright 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
 Contributed by Jean-Luc Re'my and the Spaces project, INRIA Lorraine.
 
 This file is part of the MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <limits.h>  /* For CHAR_BIT */
@@ -175,7 +175,7 @@ mpfr_zeta_pos (mpfr_t z, mpfr_srcptr s, mp_rnd_t rnd_mode)
       else
         err = ((mp_exp_t)1) << err;
       err = 1 - (-err+1); /* GET_EXP(one) - (-err+1) = err :) */
-      MPFR_FAST_COMPUTE_IF_SMALL_INPUT (z, __gmpfr_one, err, 1, rnd_mode, );
+      MPFR_FAST_COMPUTE_IF_SMALL_INPUT (z, __gmpfr_one, err, 1, rnd_mode,{});
     }
 
   d = precz + MPFR_INT_CEIL_LOG2(precz) + 10;
@@ -382,6 +382,18 @@ mpfr_zeta (mpfr_t z, mpfr_srcptr s, mp_rnd_t rnd_mode)
           mpfr_sub (s1, __gmpfr_one, s, GMP_RNDN);/* s1 = 1-s */
           mpfr_zeta_pos (z_pre, s1, GMP_RNDN);   /* zeta(1-s)  */
           mpfr_gamma (y, s1, GMP_RNDN);          /* gamma(1-s) */
+          if (MPFR_IS_INF (y)) /* Zeta(s) < 0 for -4k-2 < s < -4k,
+                                  Zeta(s) > 0 for -4k < s < -4k+2 */
+            {
+              MPFR_SET_INF (z_pre);
+              mpfr_div_2exp (s1, s, 2, GMP_RNDN); /* s/4, exact */
+              mpfr_frac (s1, s1, GMP_RNDN); /* exact, -1 < s1 < 0 */
+              if (mpfr_cmp_si_2exp (s1, -1, -1) > 0)
+                MPFR_SET_NEG (z_pre);
+              else
+                MPFR_SET_POS (z_pre);
+              break;
+            }
           mpfr_mul (z_pre, z_pre, y, GMP_RNDN);  /* gamma(1-s)*zeta(1-s) */
           mpfr_const_pi (p, GMP_RNDD);
           mpfr_mul (y, s, p, GMP_RNDN);
